#pragma once
#include <string>
#include "Filter.h"
#include "const.h"
#include "ComLogSettings.h"

//***********************************************************

class CComLogInfo
{
private:
	CFilter m_filter;
	bool m_bAutoScroll;
	bool m_bAutoStop;
	bool m_bEnableFilter;
	bool m_bFiltering[FILTER_TYPE_NUM];
	int m_logmax;
	std::string m_custom;
	CRITICAL_SECTION m_lock;
	// bNΏ: m_filter, m_custom

public:
	bool ShowFilterSettingDlg(HWND hWnd);
	void LoadSettings();

	CComLogInfo()
	{
		InitializeCriticalSection(&m_lock);
	}
	~CComLogInfo()
	{
		DeleteCriticalSection(&m_lock);
	}
	bool IsAutoScroll() const
	{
		return m_bAutoScroll;
	}
	bool IsAutoStop() const
	{
		return m_bAutoStop;
	}
	void ToggleAutoScroll()
	{
		m_bAutoScroll = ! m_bAutoScroll;
		COMLOG_SETTING.SaveAutoScrollState(m_bAutoScroll);
	}
	void ToggleAutoStop()
	{
		m_bAutoStop = ! m_bAutoStop;
		COMLOG_SETTING.SaveAutoStopState(m_bAutoStop);
	}
	bool IsFiltering(LPCTSTR cmd)
	{
		FilterList::iterator i;
		bool bFiltering = false;

		if( m_bEnableFilter ){
			Lock();
			bFiltering = m_filter.IsFiltering(cmd);
			Unlock();
		}

		return bFiltering;
	}
	int GetMaxLog()
	{
		return m_logmax;
	}

protected:
	void SetFilter();

	void Lock()
	{
		EnterCriticalSection(&m_lock);
	}
	void Unlock()
	{
		LeaveCriticalSection(&m_lock);
	}
};

//***********************************************************

