#include "StdAfx.h"
#include <utility>
#include ".\comlogmain.h"
#include "ComLogSettings.h"
#include "NsmPlugin.h"
#include "mleak.h"


//******************************************************************
// RXgN^EfXgN^
//******************************************************************

CComLogMain::CComLogMain(void)
{
	InitializeCriticalSection(&m_lock);
}

CComLogMain::~CComLogMain(void)
{
	DeleteCriticalSection(&m_lock);
}

//******************************************************************
// public
//******************************************************************

// OEChE\
void CComLogMain::ShowLogWindow(HNsmConnection hConnection)
{
	CLogWindow *wnd;

	Lock();
_RPT1(_CRT_WARN, "CComLogMain::ShowLogWindow(%d)\n", hConnection);

	wnd = m_logManager.FindLogWindow(hConnection);
	if( wnd == NULL )
	{
		wnd = m_logManager.CreateLogWindow(hConnection);
	}
	if( wnd != NULL )
	{
		wnd->Activate();
	}

	Unlock();
}

// ʐMOL^
void CComLogMain::SetComLog(HNsmConnection hConnection, LPCTSTR header, LPCTSTR log)
{
	CLogWindow *wnd;
	CConComLog *pLines;

	Lock();

	pLines = m_logManager.FindLogLines(hConnection);
_RPT3(_CRT_WARN, "CComLogMain::SetComLog(%d, %s, %s)\n", hConnection, header, log);

	if( pLines != NULL )
	{
		int logmax = COMLOG_SETTING.GetMaxLog();

		// OL^
		pLines->SetComLog(header[0], log);
		if( logmax > 0 )
		{
			pLines->Truncate(logmax);
		}

		// \XV
		wnd = m_logManager.FindLogWindow(hConnection);
		if( wnd != NULL )
		{
			wnd->UpdateLog();
		}
	}

	Unlock();
}

// OGg쐬
void CComLogMain::CreateLog(HNsmConnection hConnection, LPCSTR proto, LPCSTR acc)
{
	Lock();
_RPT1(_CRT_WARN, "CComLogMain::CreateLog(%d)\n", hConnection);
	m_logManager.CreateNewEntry(hConnection, proto, acc);
	Unlock();
}

// OGg폜
void CComLogMain::DeleteLog(HNsmConnection hConnection)
{
	Lock();
_RPT1(_CRT_WARN, "CComLogMain::DeleteLog(%d)\n", hConnection);

	m_logManager.MarkEntryDeleted(hConnection);

	Unlock();
}

// OGg폜
void CComLogMain::DeleteEntry(LPCSTR proto, LPCSTR acc)
{
	Lock();
_RPT2(_CRT_WARN, "CComLogMain::DeleteEntry(%s, %s)\n", proto, acc);

	m_logManager.DeleteEntry(proto, acc);

	Unlock();
}

void CComLogMain::AccUpdated(HNsmConnection hConnection, LPCSTR acc)
{
	Lock();
_RPT2(_CRT_WARN, "CComLogMain::AccUpdated(%d, %s)\n", hConnection, acc);

	m_logManager.AccUpdated(hConnection, acc);

	Unlock();
}

// ݒ_CAO\
void CComLogMain::ShowOptionDialog(HWND hWnd)
{
	bool bUpdated;

	bUpdated = COMLOG_SETTING.ShowFilterSettingDlg(hWnd);
	if( bUpdated )
	{
		Lock();
		m_logManager.UpdateAllEntry();
		Unlock();
	}
}
