#include "StdAfx.h"
#include ".\comlogsettings.h"
#include "const.h"
#include "util.h"
#include <algorithm>
#include "FilterSettingDlg.h"
#include "NsmPlugin.h"

const static char CLS_SECT_WINDOWRECT[]		= "WindowRect";
const static char CLS_SECT_SAVEPATH[]		= "SavePath";
const static char CLS_SECT_COMMON[]			= "Common";

const static char CLS_KEY_AUTOSCROLL[]		= "AutoScroll";
const static char CLS_KEY_AUTOSTOP[]		= "AutoStop";
const static char CLS_KEY_FILTERENABLE[]	= "FilterEnable";
const static char CLS_KEY_CUSTOMFILTER[]	= "CustomFilter";
const static char CLS_KEY_MAXLOG[]			= "MaxLog";
const static char CLS_KEY_KEEPLOG[]			= "KeepLog";
const static char CLS_KEY_RECLOG[]			= "RecLog";
const static char CLS_KEY_LOGIDR[]			= "LogDir";

CComLogSettings::CComLogSettings(LPCTSTR pluginName)
: COptionSettings(pluginName)
{
	m_bAutoScroll	= LoadAutoScrollState();
	m_bAutoStop		= LoadAutoStopState();
	m_bEnableFilter = LoadEnableFilter();
	m_bKeepLog		= LoadKeepLog();
	m_bRecLog		= LoadRecLog();
	m_logmax		= LoadMaxLog();

	LoadFilters(m_bFiltering);
	LoadCustomFilter(m_custom);
	LoadLogDir(m_dir);
	SetFilter();

	InitializeCriticalSection(&m_lock);
}

CComLogSettings::~CComLogSettings(void)
{
	DeleteCriticalSection(&m_lock);
}

void CComLogSettings::LoadLogWindowRect(LPCTSTR lpAccount, RECT &r)
{
	tstring rectStr;

	m_ini.ReadString(rectStr, CLS_SECT_WINDOWRECT, lpAccount, "");

	if( rectStr.length() == 0 ){
		LoadDefaultRect(r);
	}else{
		ParseRect(rectStr.c_str(), r);
	}
}

void CComLogSettings::SaveLogWindowRect(LPCTSTR lpAccount, RECT &r)
{
	tstring rectStr;
	char num[48];

	_itoa_s(r.left, num, 10);
	rectStr += num;
	rectStr += " ";

	_itoa_s(r.top, num, 10);
	rectStr += num;
	rectStr += " ";

	_itoa_s(r.right, num, 10);
	rectStr += num;
	rectStr += " ";

	_itoa_s(r.bottom, num, 10);
	rectStr += num;
	rectStr += " ";

	m_ini.Write(CLS_SECT_WINDOWRECT, lpAccount, rectStr.c_str());
}

void CComLogSettings::LoadDefaultRect(RECT &r)
{
	int screen_x = GetSystemMetrics(SM_CXSCREEN);
	int screen_y = GetSystemMetrics(SM_CYSCREEN);
	int width = 250;
	int height = 300;

	r.left = (screen_x - width)/2;
	r.top = (screen_y - height)/2;
	r.right = r.left + width;
	r.bottom = r.top + height;
}

void CComLogSettings::ParseRect(LPCTSTR str, RECT &r)
{
	LPCTSTR st;

	memset(&r, 0, sizeof(r));

	st = str;
	r.left = SpaceNumToken(st);
	r.top = SpaceNumToken(st);
	r.right = SpaceNumToken(st);
	r.bottom = SpaceNumToken(st);
}

int CComLogSettings::SpaceNumToken(LPCTSTR &str)
{
	LPCTSTR pos;
	int num;

	num = atoi(str);

	pos = strchr(str, ' ');
	if( pos != NULL ){
		str = pos + 1;
	}else{
		str += strlen(str);
	}

	return num;
}

void CComLogSettings::LoadSavePath(tstring &path, LPCTSTR lpAccount)
{
	m_ini.ReadString(path, CLS_SECT_SAVEPATH, lpAccount, "");
}

void CComLogSettings::SaveSavePath(LPCTSTR lpAccount, LPCTSTR path)
{
	m_ini.Write(CLS_SECT_SAVEPATH, lpAccount, path);
}

void CComLogSettings::LoadLogDir(std::string &dir)
{
	m_ini.ReadString(dir, CLS_SECT_COMMON, CLS_KEY_LOGIDR, "");
	if( dir.length() == 0 )
	{
		dir = GetMyDir();
	}
}

void CComLogSettings::SaveLogDir(const std::string &dir)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_LOGIDR, dir.c_str());
}

bool CComLogSettings::LoadAutoScrollState()
{
	return m_ini.ReadInt(CLS_SECT_COMMON, CLS_KEY_AUTOSCROLL, 1) != 0;
}

void CComLogSettings::SaveAutoScrollState(bool bState)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_AUTOSCROLL, bState);
}

bool CComLogSettings::LoadAutoStopState()
{
	return m_ini.ReadInt(CLS_SECT_COMMON, CLS_KEY_AUTOSTOP, 1) != 0;
}

void CComLogSettings::SaveAutoStopState(bool bState)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_AUTOSTOP, bState);
}

int CComLogSettings::LoadMaxLog()
{
	return m_ini.ReadInt(CLS_SECT_COMMON, CLS_KEY_MAXLOG, 0);
}

void CComLogSettings::SaveMaxLog(int value)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_MAXLOG, value);
}

bool CComLogSettings::LoadKeepLog()
{
	return m_ini.ReadBool(CLS_SECT_COMMON, CLS_KEY_KEEPLOG, true);
}

void CComLogSettings::SaveKeepLog(bool bKeep)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_KEEPLOG, bKeep);
}

bool CComLogSettings::LoadRecLog()
{
	return m_ini.ReadBool(CLS_SECT_COMMON, CLS_KEY_RECLOG, false);
}

void CComLogSettings::SaveRecLog(bool bRec)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_RECLOG, bRec);
}

bool CComLogSettings::LoadEnableFilter()
{
	return m_ini.ReadBool(CLS_SECT_COMMON, CLS_KEY_FILTERENABLE, 0);
}

void CComLogSettings::SaveEnableFilter(bool bEnable)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_FILTERENABLE, bEnable);
}

#define ITOA10_MAX_LEN		11
#define LEN_OF_STR_FILTER	6
void CComLogSettings::LoadFilters(bool bFilter[])
{
	char key[ITOA10_MAX_LEN + LEN_OF_STR_FILTER];

	strcpy_s(key, "filter");
	for( int i = 0; i < FILTER_TYPE_NUM; i++ ){
		_itoa_s(i, key + LEN_OF_STR_FILTER, 10, 10);
		bFilter[i] = m_ini.ReadInt(CLS_SECT_COMMON, key, 0) != 0;
	}
}

void CComLogSettings::SaveFilters(bool bFilter[])
{
	char key[ITOA10_MAX_LEN + LEN_OF_STR_FILTER];

	strcpy_s(key, "filter");
	for( int i = 0; i < FILTER_TYPE_NUM; i++ ){
		_itoa_s(i, key + LEN_OF_STR_FILTER, 10, 10);
		m_ini.Write(CLS_SECT_COMMON, key, bFilter[i]);
	}
}

void CComLogSettings::LoadCustomFilter(std::string &custom)
{
	m_ini.ReadString(custom, CLS_SECT_COMMON, CLS_KEY_CUSTOMFILTER, "");
}

void CComLogSettings::SaveCustomFilter(std::string &custom)
{
	m_ini.Write(CLS_SECT_COMMON, CLS_KEY_CUSTOMFILTER, custom.c_str());
}

//**************************************************

void CComLogSettings::SetFilter()
{
	m_filter.ClearFilter();
	if( m_bFiltering[FILTER_CONNECT] ){
		// T[oƂ̐ڑ
		m_filter.AddFilters("VER CVR USR XFR INF OUT");
	}
	if( m_bFiltering[FILTER_PNG] ){
		// T[oƂ̐ڑmF
		m_filter.AddFilters("PNG QNG CHL QRY");
	}
	if( m_bFiltering[FILTER_MEMBERLIST] ){
		// oXg
		m_filter.AddFilters("SYN GTC BLP LSG LST PRP BPR ADD REM ADG RMG REG");
	}
	if( m_bFiltering[FILTER_MEMBERSTATE] ){
		// oԕύX
		m_filter.AddFilters("CHG REA NLN FLN ILN");
	}
	if( m_bFiltering[FILTER_SESSION] ){
		// ZbV
		m_filter.AddFilters("CAL RNG ANS IRO JOI BYE FIL TFR CCL");
	}
	if( m_bFiltering[FILTER_MSG] ){
		// bZ[W
		m_filter.AddFilters("MSG ACK NAK");
	}
	if( m_bFiltering[FILTER_ETC] ){
		// ̑
		m_filter.AddFilters("URL NOT SND SDC FND PAG IPG");
	}
	if( m_bFiltering[FILTER_ERR] ){
		// G[
		m_filter.AddErrorFilter();
	}
	if( m_bFiltering[FILTER_CUSTOM] ){
		// JX^
		m_filter.AddFilters(m_custom.c_str());
	}
}

bool CComLogSettings::ShowFilterSettingDlg(HWND hWnd)
{
	CFilterSettingDlg dlg;

	Lock();
	dlg.SetMaxLog(m_logmax);
	dlg.SetFilter(m_bFiltering);
	dlg.SetEnableFilter(m_bEnableFilter);
	dlg.SetCustomString(m_custom.c_str());
	dlg.SetRecLog(m_bRecLog);
	dlg.SetKeepLog(m_bKeepLog);
	dlg.SetLogDir(m_dir.c_str());
	Unlock();

	if( dlg.DoModal(CNsmPlugin::GetModuleInstance(), hWnd) != IDOK )
	{
		return false;
	}

	Lock();
	m_logmax = dlg.GetMaxLog();
	m_bEnableFilter = dlg.GetEnableFilter();
	m_custom = dlg.GetCustomString();
	m_bRecLog = dlg.GetRecLog();
	m_bKeepLog = dlg.GetKeepLog();
	m_dir = dlg.GetLogDir();
	dlg.GetFilter(m_bFiltering);
	SetFilter();

	SaveMaxLog(m_logmax);
	SaveEnableFilter(m_bEnableFilter);
	SaveFilters(m_bFiltering);
	SaveCustomFilter(m_custom);
	SaveKeepLog(m_bKeepLog);
	SaveRecLog(m_bRecLog);
	SaveLogDir(m_dir);
	Unlock();

	return true;
}
