#pragma once
#include "..\nsmsgs\optionsettings.h"
#include <string>
#include "Filter.h"
#include "const.h"

#define COMLOG_SETTING	(CComLogSettings::GetInstance())

class CComLogSettings :
	public COptionSettings
{
// singleton -->
private:
	static CComLogSettings m_singleton;

private:
	CComLogSettings(LPCTSTR pluginName);
	~CComLogSettings(void);

public:
	static CComLogSettings& GetInstance()
	{
		return m_singleton;
	}
// <-- singleton

protected:
	CFilter m_filter;
	bool m_bAutoScroll;
	bool m_bAutoStop;
	bool m_bEnableFilter;
	bool m_bFiltering[FILTER_TYPE_NUM];
	bool m_bRecLog;
	bool m_bKeepLog;
	int m_logmax;
	std::string m_custom;
	std::string m_dir;
	CRITICAL_SECTION m_lock;
	// bNΏ: m_filter, m_custom

public:
	void LoadLogWindowRect(LPCTSTR lpAccount, RECT &r);
	void SaveLogWindowRect(LPCTSTR lpAccount, RECT &r);
	void LoadSavePath(tstring &path, LPCTSTR lpAccount);
	void SaveSavePath(LPCTSTR lpAccount, LPCTSTR path);

	bool LoadAutoScrollState();
	void SaveAutoScrollState(bool bState);
	bool LoadAutoStopState();
	void SaveAutoStopState(bool bState);
	bool LoadEnableFilter();
	void SaveEnableFilter(bool bEnable);

	bool ShowFilterSettingDlg(HWND hWnd);

	bool IsKeepLogEnable()
	{
		return m_bKeepLog;
	}
	bool IsRecLogEnable()
	{
		return m_bRecLog;
	}
	bool IsAutoScroll() const
	{
		return m_bAutoScroll;
	}
	bool IsAutoStop() const
	{
		return m_bAutoStop;
	}
	void ToggleAutoScroll()
	{
		m_bAutoScroll = ! m_bAutoScroll;
		COMLOG_SETTING.SaveAutoScrollState(m_bAutoScroll);
	}
	void ToggleAutoStop()
	{
		m_bAutoStop = ! m_bAutoStop;
		COMLOG_SETTING.SaveAutoStopState(m_bAutoStop);
	}
	bool IsFiltering(LPCTSTR cmd)
	{
		FilterList::iterator i;
		bool bFiltering = false;

		if( m_bEnableFilter ){
			Lock();
			bFiltering = m_filter.IsFiltering(cmd);
			Unlock();
		}

		return bFiltering;
	}
	int GetMaxLog()
	{
		return m_logmax;
	}
	void GetLogDir(std::string &dir)
	{
		dir = m_dir;
		if( dir[dir.length() - 1] != '\\' )
		{
			dir += '\\';
		}
	}

protected:
	int LoadMaxLog();
	void SaveMaxLog(int max);
	bool LoadKeepLog();
	void SaveKeepLog(bool bKeep);
	bool LoadRecLog();
	void SaveRecLog(bool bRec);
	void LoadLogDir(std::string &dir);
	void SaveLogDir(const std::string &dir);

	void LoadFilters(bool bFilter[]);
	void SaveFilters(bool bFilter[]);
	void LoadCustomFilter(std::string &custom);
	void SaveCustomFilter(std::string &custom);

	void LoadDefaultRect(RECT &r);
	void ParseRect(LPCTSTR str, RECT &r);
	int  SpaceNumToken(LPCTSTR &str);
	void SetFilter();

	void Lock()
	{
		EnterCriticalSection(&m_lock);
	}
	void Unlock()
	{
		LeaveCriticalSection(&m_lock);
	}
};
