#include "StdAfx.h"
#include ".\concomlog.h"
#include "cs_conv.h"
#include <fstream>
#include <iomanip>
#include "ComLogSettings.h"
#include "util.h"

CConComLog::CConComLog(const std::string* proto, const std::string* acc)
	: m_nMultiLineLen(0)
	, m_log_st(0)
	, m_pProto(proto)
	, m_pAcc(acc)
	, m_bMsnp8(strcmp("MSNP8", proto->c_str()) == 0)
{
	UpdateFilePath();
	InitializeCriticalSection(&m_lock);
}

CConComLog::~CConComLog(void)
{
	DeleteCriticalSection(&m_lock);
	LogLineList::iterator i;

	for( i = m_lines.begin(); i != m_lines.end(); i++ ){
		delete *i;
	}
}


// OnLog1񕪂̃O󂯎A_sƂAddLogɓn
void CConComLog::SetComLog(char header, LPCSTR log)
{
	LPCTSTR pos, st;
	int len;

_RPT1(_CRT_WARN, "SetComLog: %s", log);
	st = log;

	Lock();
	for(;;)
	{
		pos = strstr(st, "\r\n");
		len = (pos == NULL) ? strlen(st): pos - st;

		if( m_bMsnp8 && (m_nMultiLineLen > 0) )
		{
			if( len <= m_nMultiLineLen )
			{
				AddLine(false, header, st, len);
				m_nMultiLineLen -= len;
				if( pos != NULL )
				{
					m_nMultiLineLen -= 2;
					if( m_nMultiLineLen == 0 )
					{
						AddLine(false, header, "", 0);
					}
				}
				DEBUG_CODE( else _ASSERT(m_nMultiLineLen == 0); );
			}
			else
			{
				AddLine(false, header, st, m_nMultiLineLen);
				AddLine(true, header, st+m_nMultiLineLen, len-m_nMultiLineLen);
			}
		}
		else
		{
			AddLine(true, header, st, len);
		}

		if( (pos == NULL) || (st[len+2] == '\0') )
		{
			break;
		}

		st = pos + 2;
	}
	Unlock();
}

void CConComLog::Truncate(int max)
{
	while( m_lines.size() > (unsigned int)max )
	{
		LogLineList::iterator st = m_lines.begin();
		LogLineList::iterator ed = st;
		ed++;

		if( (*st)->IsFirstLine() )
		{
			while( (ed != m_lines.end()) && (! (*ed)->IsFirstLine()) )
			{
				++ed;
			}
		}

		LogLineList::iterator i;
		for( i = st; i != ed; ++i )
		{
			delete *i;
			m_log_st++;
		}

		m_lines.erase(st, ed);
	}
}


// _s󂯎胍OXgɒǉB
void CConComLog::AddLine(bool bFirstLine, char header, LPCSTR log, int len)
{
	_ASSERT( len >= 0 );
	CLogLine *line;
	DynamicArray<char> buf(len+1);
	std::string utf8(log, len);
	std::wstring uni;
	std::string sjis;
	char timestamp[32];

	{
		SYSTEMTIME st;
		GetLocalTime(&st);

		sprintf_s(timestamp, "%02d:%02d:%02d", st.wHour, st.wMinute, st.wSecond);
	}

	// SJISɕϊ
	{
		::UTF8toUnicode(utf8.c_str(), uni);
		::UnicodeToSJIS(uni.c_str(), sjis);
	}

	// Oo
	if( COMLOG_SETTING.IsRecLogEnable() )
	{
		std::fstream fs(m_file.c_str(), std::ios::out | std::ios::app );

		if( ! fs.fail() )
		{
			if( bFirstLine )
			{
				fs << timestamp << "\t" << header << "\t" << sjis << std::endl;
			}
			else
			{
				fs << "        " << "\t" << " " << "\t" << sjis << std::endl;
			}
		}
	}

	// loglineǉ
	{
		line = new CLogLine();
		line->Set(timestamp, header, sjis.c_str(), bFirstLine, m_log_st + m_lines.size());
		m_lines.push_back(line);
	}

	if( m_bMsnp8 && bFirstLine )
	{
		m_nMultiLineLen = GetMultiLineLen(utf8.c_str());
	}
}

int CConComLog::GetMultiLineLen(LPCTSTR line)
{
	LPCTSTR first, last;
	char *end;
	int size;

	first = strchr(line, ' ');
	last = strrchr(line, ' ');
	if( first == last ) return 0;

	size = strtol(last+1, &end, 10);
	if( *end != '\0' ) return 0;

	if( memcmp(line, "MSG", 3) == 0 ) return size;
	if( memcmp(line, "QRY", 3) == 0 ) return size;
	if( memcmp(line, "PAG", 3) == 0 ) return size;
	if( memcmp(line, "NOT", 3) == 0 ) return size;
	if( memcmp(line, "IPG", 3) == 0 ) return size;
	return 0;
}

void CConComLog::UpdateFilePath()
{
	COMLOG_SETTING.GetLogDir(m_file);

	m_file += *m_pProto;
	m_file += "\\";

	::CreateDirAll(m_file.c_str());

	m_file += *m_pAcc;
	m_file += ".txt";
}

//**************************************************************

