#include "StdAfx.h"
#include ".\filtersettingdlg.h"
#include "resource.h"
#include <Shlobj.h>
#include <Shtypes.h>

CFilterSettingDlg::CFilterSettingDlg(void)
: CDlg(IDD_FILTERSETTING)
{
}

CFilterSettingDlg::~CFilterSettingDlg(void)
{
}

BOOL CFilterSettingDlg::DlgProc(UINT uMsg, WPARAM wParam, LPARAM /*lParam*/)
{
	switch( uMsg ){
	case WM_COMMAND:
		switch( LOWORD(wParam) ){
		case IDC_USE_FILTER:
			OnClickUseFilter();
			break;
		case IDC_FILTER_CUSTOM:
			OnClickFilterCustom();
			break;
		case IDC_REC_FILE:
			OnClickRecLog();
			break;
		case IDC_LOG_DIR_REF:
			OnClickLogDirRef();
			break;
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}

	return TRUE;
}

BOOL CFilterSettingDlg::InitDlgCtrl()
{
	// MSNP8 filter
	{
		SetCheck(IDC_USE_FILTER, m_bEnableFilter);

		SetCheck(IDC_FILTER_CONNECT, m_bFiltering[FILTER_CONNECT]);
		SetCheck(IDC_FILTER_PNG, m_bFiltering[FILTER_PNG]);
		SetCheck(IDC_FILTER_MEMBERLIST, m_bFiltering[FILTER_MEMBERLIST]);
		SetCheck(IDC_FILTER_MEMBERSTATE, m_bFiltering[FILTER_MEMBERSTATE]);
		SetCheck(IDC_FILTER_SESSION, m_bFiltering[FILTER_SESSION]);
		SetCheck(IDC_FILTER_MSG, m_bFiltering[FILTER_MSG]);
		SetCheck(IDC_FILTER_ETC, m_bFiltering[FILTER_ETC]);
		SetCheck(IDC_FILTER_ERR, m_bFiltering[FILTER_ERR]);
		SetCheck(IDC_FILTER_CUSTOM, m_bFiltering[FILTER_CUSTOM]);

		SetItemText(IDC_CUSTOM, GetCustomString());
	}

	// other function
	{
		SetItemText(IDC_MAXLOG, m_logmax);

		SetCheck(IDC_KEEP_LOG, m_bKeepLog);
		SetCheck(IDC_REC_FILE, m_bRecLog);
		SetItemText(IDC_LOG_DIR, m_logdir.c_str());
	}

	EnableAll(m_bEnableFilter);
	UpdateLogDirEnable();

	return TRUE;
}

BOOL CFilterSettingDlg::GetDlgCtrlValue()
{
	// MSNP8 filter
	{
		m_bEnableFilter = (GetCheck(IDC_USE_FILTER) != FALSE);

		m_bFiltering[FILTER_CONNECT] = GetCheck(IDC_FILTER_CONNECT);
		m_bFiltering[FILTER_PNG] = GetCheck(IDC_FILTER_PNG);
		m_bFiltering[FILTER_MEMBERLIST] = GetCheck(IDC_FILTER_MEMBERLIST);
		m_bFiltering[FILTER_MEMBERSTATE] = GetCheck(IDC_FILTER_MEMBERSTATE);
		m_bFiltering[FILTER_SESSION] = GetCheck(IDC_FILTER_SESSION);
		m_bFiltering[FILTER_MSG] = GetCheck(IDC_FILTER_MSG);
		m_bFiltering[FILTER_ETC] = GetCheck(IDC_FILTER_ETC);
		m_bFiltering[FILTER_ERR] = GetCheck(IDC_FILTER_ERR);
		m_bFiltering[FILTER_CUSTOM] = GetCheck(IDC_FILTER_CUSTOM);

		GetItemText(IDC_CUSTOM, m_custom);
	}

	// other function
	{
		m_logmax = GetItemTextInt(IDC_MAXLOG);

		m_bKeepLog = GetCheck(IDC_KEEP_LOG);
		m_bRecLog = GetCheck(IDC_REC_FILE);
		GetItemText(IDC_LOG_DIR, m_logdir);
	}

	return TRUE;
}

void CFilterSettingDlg::OnClickUseFilter()
{
	EnableAll(GetCheck(IDC_USE_FILTER));
}

void CFilterSettingDlg::EnableAll(BOOL bEnable)
{
	SetEnable(IDC_FILTER_CONNECT, bEnable);
	SetEnable(IDC_FILTER_PNG, bEnable);
	SetEnable(IDC_FILTER_MEMBERLIST, bEnable);
	SetEnable(IDC_FILTER_MEMBERSTATE, bEnable);
	SetEnable(IDC_FILTER_SESSION, bEnable);
	SetEnable(IDC_FILTER_MSG, bEnable);
	SetEnable(IDC_FILTER_ETC, bEnable);
	SetEnable(IDC_FILTER_ERR, bEnable);
	SetEnable(IDC_FILTER_CUSTOM, bEnable);
	SetEnable(IDC_CUSTOM, bEnable && GetCheck(IDC_FILTER_CUSTOM));
}

void CFilterSettingDlg::OnClickFilterCustom()
{
	SetEnable(IDC_CUSTOM, GetCheck(IDC_FILTER_CUSTOM));
}

void CFilterSettingDlg::OnClickRecLog()
{
	UpdateLogDirEnable();
}

void CFilterSettingDlg::OnClickLogDirRef()
{
	tstring edit_path;
	TCHAR lpSelectedPath[MAX_PATH];
	BROWSEINFO bi;
	LPITEMIDLIST lpSelectedID;
	LPMALLOC lpSHMalloc;

	// IvV_CAÕGfBbg{bNXpX擾
	GetItemText(IDC_LOG_DIR, edit_path);

	if( FAILED(::SHGetMalloc(&lpSHMalloc)) )
	{
        return;
	}

	// tH_I_CAOJ
	bi.hwndOwner = m_hDlgWnd;
	bi.pidlRoot = NULL;
	bi.pszDisplayName = NULL;
	bi.lpszTitle = _T("OۑtH_I");
	bi.ulFlags = (BIF_NEWDIALOGSTYLE);
	bi.lpfn = BrowsDlgProc;
	bi.lParam = (LPARAM)edit_path.c_str();
	bi.iImage = 0;

	lpSelectedID = ::SHBrowseForFolder(&bi);

	if( lpSelectedID ){
        ::SHGetPathFromIDList(lpSelectedID, lpSelectedPath);
		lpSHMalloc->Free(lpSelectedID);

		// IvV_CAÕGfBbg{bNXփpXݒ
		SetItemText(IDC_LOG_DIR, lpSelectedPath);
	}
}

void CFilterSettingDlg::UpdateLogDirEnable()
{
	bool bEnable = GetCheck(IDC_REC_FILE);

	SetEnable(IDC_LOG_DIR, bEnable);
	SetEnable(IDC_LOG_DIR_REF, bEnable);
}


//*******************************************************************
// _CAOvV[W
//*******************************************************************
// tH_I_CAÕ_CAOvV[W
int CALLBACK CFilterSettingDlg::BrowsDlgProc(HWND hWnd, UINT uMsg, LPARAM /*lParam*/, LPARAM lpData)
{
    switch(uMsg){
    case BFFM_INITIALIZED:
        if((TCHAR*)lpData != NULL)
		{
			::SendMessage(hWnd, BFFM_SETSELECTION, (WPARAM)TRUE, (LPARAM)lpData);
		}
        break;
    }

    return 0;
}
