#include "StdAfx.h"
#include <process.h>
#include <Commdlg.h>
#include <algorithm>
#include "resource.h"
#include ".\logwindow.h"
#include "CommonService.h"
#include "util.h"
#include "mleak.h"
#include "ComLogPlugin.h"

#define CLW_WIDTH_SENDER	12
#define CLW_WIDTH_TIME		60
#define CLW_WIDTH_MSG		1000
#define CLW_TITLE_PREFIX	"ʐMO("
#define CLW_TITLE_POSTFIX	")"

WNDPROC CLogWindow::m_listOrgProc;
int CLogWindow::m_iSelectStart;
int CLogWindow::m_iPreSelect;

//**********************************************************************
// RXgN^EfXgN^
//**********************************************************************

CLogWindow::CLogWindow(const std::string* proto, const std::string* acc, CConComLog* pLines)
: CDlg(IDD_LOGDLG, IDR_ACCELERATOR1)
, m_bDeleteted(false)
, m_bDlgShowing(FALSE)
, m_bFiltering(false)
, m_pProto(proto)
, m_pAcc(acc)
, m_pLines(pLines)
{
	m_title = *proto;
	m_title += "-";
	m_title += *acc;
	COMLOG_SETTING.LoadSavePath(m_savepath, acc->c_str());
	COMLOG_SETTING.LoadLogWindowRect(m_title.c_str(), m_rect);
}

CLogWindow::~CLogWindow(void)
{
	if( m_bDlgShowing )
	{
		DestroyWindow();
	}
	COMLOG_SETTING.SaveLogWindowRect(m_title.c_str(), m_rect);
	COMLOG_SETTING.SaveSavePath(m_title.c_str(), m_savepath.c_str());
}

//**********************************************************************
// _CAOx[X
//**********************************************************************

BOOL CLogWindow::InitDlgCtrl()
{
	LVCOLUMN lvc;
	HWND hListWnd;

	::SetWindowText(m_hDlgWnd,
		(CLW_TITLE_PREFIX + m_title + CLW_TITLE_POSTFIX).c_str());
	::MoveWindow(m_hDlgWnd, m_rect.left, m_rect.top,
		m_rect.right - m_rect.left, m_rect.bottom - m_rect.top, FALSE);

	hListWnd = GetDlgItem(IDC_LOGLIST);
	m_listOrgProc = (WNDPROC)GetWindowLongPtr(hListWnd, GWL_WNDPROC);
	SetWindowLongPtr(hListWnd, GWL_WNDPROC, (LONG_PTR)ListCtrlProc);

	ResetListView(hListWnd);
	ListView_SetExtendedListViewStyle(hListWnd,
		LVS_EX_FULLROWSELECT | LVS_EX_LABELTIP );

	lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_SUBITEM;
	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = CLW_WIDTH_SENDER;
	lvc.iSubItem = 0;
	ListView_InsertColumn(hListWnd, 0, &lvc);

	lvc.cx = CLW_WIDTH_TIME;
	lvc.iSubItem = 1;
	ListView_InsertColumn(hListWnd, 1, &lvc);

	lvc.cx = CLW_WIDTH_MSG;
	lvc.iSubItem = 2;
	ListView_InsertColumn(hListWnd, 2, &lvc);

	OnLogUpdated();

	return TRUE;
}
/*
void CLogWindow::GetDlgCtrlValue()
{
}
*/
BOOL CLogWindow::DlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg){
	case WM_USER_LOGUPDATED:
		OnLogUpdated();
		break;
	case WM_USER_FILTERUPDATED:
		OnFilterUpdated();
		break;
	case WM_SIZE:
		OnSize(wParam, LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_MOVE:
		OnMove(LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_CONTEXTMENU:
		OnContextMenu((HWND)wParam, LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_CLOSE:
		OnClose();
		return FALSE;
	case WM_DESTROY:
		OnDestroy();
		break;
	case WM_COMMAND:
		switch( LOWORD(wParam) ){
		HANDLE_CMD(IDM_COPY, OnCopy());
		HANDLE_CMD(IDM_SELECTALL, OnSelectAll());
		HANDLE_CMD(IDM_ALLCLEAR, OnAllClear());
		HANDLE_CMD(IDM_SAVE, OnSave());
		HANDLE_CMD(IDM_TOGGLEAUTOSCROLL, OnToggleAutoScroll());
		HANDLE_CMD(IDM_TOGGLEAUTOSTOP, OnToggleAutoStop());
		HANDLE_CMD(IDM_FILTERSETTING, OnFilterSetting());
		default:
			return FALSE;
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

//**********************************************************************
// LogWindow쐬ƕ\
//**********************************************************************

void CLogWindow::Create(HINSTANCE hInstance)
{
	m_hInstance = hInstance;
	ShowDlg();
}

void CLogWindow::ShowDlg()
{
	if( ! m_bDlgShowing ){
		m_bDlgShowing = true;
		CDlg::Create(m_hInstance, NULL);
	}
	::ShowWindow(m_hDlgWnd, SW_RESTORE);
	::SetForegroundWindow(m_hDlgWnd);
}

void CLogWindow::DestroyWindow()
{
	::SendMessage(m_hDlgWnd, WM_COMMAND, IDCANCEL, 0);
}

void CLogWindow::Activate()
{
	ShowDlg();
}

void CLogWindow::MarkDeleted()
{
	m_bDeleteted = true;
	if( ! m_bDlgShowing )
	{
		DeleteThisEntry();
	}
}

void CLogWindow::DeleteThisEntry()
{
	CComLogPlugin::GetInstance().DoDeleteEntry(
		m_pProto->c_str(),
		m_pAcc->c_str()
	);
}

//**********************************************************************
// LogWindow
//**********************************************************************

void CLogWindow::UpdateLog()
{
	if( m_bDlgShowing ){
		::PostMessage(m_hDlgWnd, WM_USER_LOGUPDATED, 0, 0);
	}
}

void CLogWindow::UpdateFilter()
{
	if( m_bDlgShowing ){
		::PostMessage(m_hDlgWnd, WM_USER_FILTERUPDATED, 0, 0);
	}
}

//**********************************************************************
// O\
//**********************************************************************

void CLogWindow::OnLogUpdated(bool bFull/*=false*/)
{
	HWND hListWnd = GetDlgItem(IDC_LOGLIST);
	bool bAutoScroll = false;

	if( COMLOG_SETTING.IsAutoScroll() )
	{
		// I[gXgbvȂXN[Ȃ
		if( COMLOG_SETTING.IsAutoStop() )
		{
			const int showTop = ListView_GetTopIndex(hListWnd);
			const int showCnt = ListView_GetCountPerPage(hListWnd);
			const int showAll = ListView_GetItemCount(hListWnd);

			if( showAll <= showTop + showCnt )
			{
				bAutoScroll = true;
			}
		}
		else
		{
			bAutoScroll = true;
		}
	}

	// \XVp update check
	const unsigned int uc_line_st = m_showedLogSt;
	const unsigned int uc_line_ed = m_showedLogEd;

	// \XV
	if( bFull )
	{
		ResetListView(hListWnd);
	}
	UpdateListView(hListWnd);

	const unsigned int line_st = m_showedLogSt;
	const unsigned int line_ed = m_showedLogEd;

	if( bAutoScroll )
	{
		// \XVꂽI[gXN[
		if( (line_ed != uc_line_ed) || (line_st != uc_line_st) )
		{
			const int showAll = ListView_GetItemCount(hListWnd);
			ListView_EnsureVisible(hListWnd, showAll - 1, FALSE);
		}
	}
}

void CLogWindow::ResetListView(HWND hListWnd)
{
	m_showedLogSt = 0;
	m_showedLogEd = 0;
	ListView_DeleteAllItems(hListWnd);
}

void CLogWindow::UpdateListView(HWND hListWnd)
{
	// ObN
	CConComLog::LogLock lines(m_pLines);
	const unsigned int log_st = m_pLines->GetLogStartNo();

	// ListView ̏߂폜
	{
		LVITEM item;

		item.mask = LVIF_PARAM;
		item.iSubItem = 0;
		item.iItem = ListView_GetNextItem(hListWnd, -1, LVNI_ALL);

		while( item.iItem >= 0 )
		{
			ListView_GetItem(hListWnd, &item);
			if( log_st <= (unsigned int)item.lParam )
			{
				break;
			}
			DeleteLine(hListWnd, 0);
			item.iItem = ListView_GetNextItem(hListWnd, -1, LVNI_ALL);
		}

		m_showedLogSt = log_st;
	}

	// ListView ̏Iɒǉ
	{
		unsigned int line_num = max((int)(m_showedLogEd - m_showedLogSt), 0);
		CConComLog::LogLineList::const_iterator i = ::ListGetAt(*lines, line_num);

		for( ; i != lines->end(); ++i )
		{
			if( m_pLines->IsMsnp8() )
			{
				if( (*i)->IsFirstLine() )
				{
					m_bFiltering = COMLOG_SETTING.IsFiltering((*i)->GetMsg());
				}
				if( m_bFiltering )
				{
					continue;
				}
			}

			InsertLine(hListWnd, line_num++, (*i)->GetNo(), **i);
		}

		m_showedLogEd = m_pLines->GetLogEndNo() + 1;
	}
}

void CLogWindow::OnFilterUpdated()
{
	if( m_pLines->IsMsnp8() )
	{
		OnLogUpdated(true);
	}
}

void CLogWindow::InsertLine(HWND hListWnd, int nPos, int no, const CLogLine &line)
{
	LVITEM lvi;

	lvi.mask = LVIF_PARAM;

	lvi.iItem = nPos;
	lvi.iSubItem = 0;
	lvi.lParam = no;
	ListView_InsertItem(hListWnd, &lvi);
	ListView_SetItemText(hListWnd, nPos, 0, (LPSTR)line.GetSender());
	ListView_SetItemText(hListWnd, nPos, 1, (LPSTR)line.GetTime());
	ListView_SetItemText(hListWnd, nPos, 2, (LPSTR)line.GetMsg());
}

//**********************************************************************
// Cxgnh
//**********************************************************************

void CLogWindow::OnClose()
{
	m_bDlgShowing = FALSE;
}

void CLogWindow::OnDestroy(){
	if( m_bDeleteted )
	{
		DeleteThisEntry();
	}
}

void CLogWindow::OnSize(int nType, int /*width*/, int /*height*/)
{
	switch( nType ){
	case SIZE_MAXIMIZED:
	case SIZE_RESTORED:
		RECT r;
		HWND hList;

		hList = GetDlgItem(IDC_LOGLIST);
		GetClientRect(m_hDlgWnd, &r);
		MoveWindow(hList, 0, 0, r.right, r.bottom, TRUE);

		UpdateWindowRect();
		break;
	}
}

void CLogWindow::OnMove(int /*x*/, int /*y*/)
{
	if( ! IsIconic(m_hDlgWnd) )
		UpdateWindowRect();
}

//**********************************************************************
// ReLXgj[
//**********************************************************************

void CLogWindow::OnContextMenu(HWND /*hWnd*/, int x, int y)
{
	HMENU hMenu, hPopup;
	MENUITEMINFO mii;

	hMenu = LoadMenu(m_hInstance, MAKEINTRESOURCE(IDR_POPMENU));
	hPopup = GetSubMenu(hMenu, 0);

	// AutoScroll check-mark
	memset(&mii, 0, sizeof(mii));
	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_STATE;
	mii.fState = COMLOG_SETTING.IsAutoScroll() ? MFS_CHECKED: MFS_UNCHECKED;
	::SetMenuItemInfo(hPopup, IDM_TOGGLEAUTOSCROLL, FALSE, &mii);

	// AutoStop check-mark
	mii.fState = 
		(COMLOG_SETTING.IsAutoStop() ? MFS_CHECKED: MFS_UNCHECKED)
		| (COMLOG_SETTING.IsAutoScroll() ? MFS_ENABLED: MFS_DISABLED);
	::SetMenuItemInfo(hPopup, IDM_TOGGLEAUTOSTOP, FALSE, &mii);

	::TrackPopupMenu(hPopup,
		(TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON),
		x, y, 0, m_hDlgWnd, 0);
	DestroyMenu(hMenu);
}

void CLogWindow::OnCopy()
{
	COnCopyFunc func;

	ForEachListItem(LVNI_SELECTED, func);
	CopyToClipboard(func.GetStr());
}

void CLogWindow::CopyToClipboard(LPCTSTR str)
{
	HGLOBAL hMem = NULL;
	LPTSTR pMem;

	if( ! ::OpenClipboard(m_hDlgWnd) ) return;

	if( ::EmptyClipboard() ){
		hMem = ::GlobalAlloc(GMEM_MOVEABLE, lstrlen(str) + 1);

		pMem = (LPTSTR)::GlobalLock(hMem);
		lstrcpy(pMem, str);
		::GlobalUnlock(hMem);
		::SetClipboardData(CF_TEXT, hMem);
	}
	::CloseClipboard();
	if( hMem != NULL ) ::GlobalFree(hMem);
}

void CLogWindow::OnSelectAll()
{
	ListView_SetItemState(GetDlgItem(IDC_LOGLIST), -1, LVIS_SELECTED, LVIS_SELECTED);
}

void CLogWindow::OnSave()
{
	OPENFILENAME ofn;
	TCHAR buf[MAX_PATH];
//	HWND hList = GetDlgItem(IDC_LOGLIST);
	FILE *f;
	std::string str;

	_tcsncpy_s(buf, m_savepath.c_str(), MAX_PATH-1);

	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = m_hDlgWnd;
	ofn.lpstrFilter = "eLXgt@C(.txt)\0*.txt\0";
	ofn.lpstrFile = buf;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = (OFN_ENABLESIZING|OFN_OVERWRITEPROMPT);

	if( GetSaveFileName(&ofn) ){
		m_savepath = buf;

		fopen_s(&f, buf, "wb");
		if( f == NULL ){
			_ASSERT(FALSE);
			return;
		}

		COnSaveFunc func(f);
		ForEachListItem(LVNI_ALL, func);
		fclose(f);
	}
}

void CLogWindow::OnAllClear()
{
	{
		CConComLog::LogLock lines(m_pLines);
		CConComLog::LogLineList::iterator i;

		for( i = lines->begin(); i != lines->end(); i++ )
			delete *i;
		lines->clear();
	}

	ListView_DeleteAllItems(GetDlgItem(IDC_LOGLIST));
}

void CLogWindow::OnFilterSetting()
{
	_beginthread(FilterSettingDlgThread, 0, m_hDlgWnd);
}

void __cdecl CLogWindow::FilterSettingDlgThread(void* hWnd)
{
	NSM_SERVICE.CallService(CLP_SERVICE_SHOW_OPTION_DLG, (WPARAM)hWnd);
}

template <class Func>
void CLogWindow::ForEachListItem(UINT flags, Func &f)
{
	LVITEM item;
	HWND hList = GetDlgItem(IDC_LOGLIST);

	item.mask = LVIF_PARAM;
	item.iSubItem = 0;
	{
		CConComLog::LogLock lines(m_pLines);
		CConComLog::LogLineList::iterator i = lines->begin();

		item.iItem = ListView_GetNextItem(hList, -1, flags);
		while( item.iItem >= 0 )
		{
			ListView_GetItem(hList, &item);
			while( (*i)->GetNo() < item.lParam )
			{
				++i;
			}
			_ASSERT( (*i)->GetNo() == item.lParam );
			if( (*i)->GetNo() == item.lParam )
			{
				f(*i);
			}
			item.iItem = ListView_GetNextItem(hList, item.iItem, flags);
		}
	}
}

int CLogWindow::LogListHitTest(HWND hWnd, int x, int y)
{
	LVHITTESTINFO info;

	info.pt.x = x;
	info.pt.y = y;
	info.flags = LVHT_ONITEM;
	return ListView_HitTest(hWnd, &info);
}

//**********************************************************************
// TuNX
//**********************************************************************

LRESULT CALLBACK CLogWindow::ListCtrlProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg){
	case WM_MOUSEMOVE:
		if( wParam & MK_LBUTTON ){
			int iItem = LogListHitTest(hWnd, LOWORD(lParam), HIWORD(lParam));
			if( iItem != -1 ){
				int sel_st=0, sel_ed=-1, unsel_st=0, unsel_ed=-1;

				if( m_iPreSelect < iItem ){
					if( iItem < m_iSelectStart ){				// pre < now < st
						unsel_st = m_iPreSelect; unsel_ed = iItem - 1;
					}else if( m_iPreSelect < m_iSelectStart ){	// pre < st < now
						unsel_st = m_iPreSelect; unsel_ed = m_iSelectStart - 1;
						sel_st = m_iSelectStart + 1; sel_ed = iItem;
					}else{										// st < pre < now
						sel_st = m_iPreSelect + 1; sel_ed = iItem;
					}
				}else{
					if( m_iPreSelect < m_iSelectStart ){	// now < pre < st
						sel_st = iItem; sel_ed = m_iPreSelect - 1;
					}else if( iItem < m_iSelectStart ){		// now < st < pre
						unsel_st = m_iSelectStart + 1; unsel_ed = m_iPreSelect;
						sel_st = iItem; sel_ed = m_iSelectStart - 1;
					}else{									// st < now < pre
						unsel_st = iItem + 1; unsel_ed = m_iPreSelect;
					}
				}
				for( int i = unsel_st; i <= unsel_ed; i++ )
					ListView_SetItemState(hWnd, i, 0, LVIS_SELECTED);
				for( int i = sel_st; i <= sel_ed; i++ )
					ListView_SetItemState(hWnd, i, LVIS_SELECTED, LVIS_SELECTED);

				m_iPreSelect = iItem;
			}
		}
		break;
	case WM_LBUTTONDOWN:
		m_iSelectStart = LogListHitTest(hWnd, LOWORD(lParam), HIWORD(lParam));
		m_iPreSelect = m_iSelectStart;
		::SetCapture(hWnd);
		break;
	case WM_LBUTTONUP:
		::ReleaseCapture();
		m_iSelectStart = -1;
		break;
	}

	return CallWindowProc(m_listOrgProc, hWnd, msg, wParam, lParam);
}

