#pragma once
#include <string>
#include "../nsmsgs/Dlg.h"
#include "ConComLog.h"
#include "ComLogInfo.h"
#include "LogOperateInfo.h"
#include "ComLogSettings.h"

//*************************************************************

class COnCopyFunc
{
private:
	std::string m_str;

public:
	LPCTSTR GetStr() const
	{
		return m_str.c_str();
	}
	void operator()(CLogLine *line)
	{
		m_str += line->GetFormattedLine();
	}
};

//*************************************************************

class COnSaveFunc
{
private:
	FILE* m_file;

public:
	COnSaveFunc(FILE* f)
	{
		m_file = f;
	}
	void operator()(CLogLine *line)
	{
		std::string str = line->GetFormattedLine();

		fwrite(str.c_str(), 1, str.length(), m_file);
	}
};

//*************************************************************

class CLogWindow
	: public CDlg
{
private:
	// value
	unsigned int m_showedLogSt;	// \O̍ŏVAID
	unsigned int m_showedLogEd;	// \O̍őVAID+1
	const std::string* m_pProto;
	const std::string* m_pAcc;
	CConComLog* const m_pLines;
	static WNDPROC m_listOrgProc;

	// state				// modify
	RECT m_rect;			// Create, UpdateWindowRect
	bool m_bDlgShowing;
	bool m_bFiltering;
	bool m_bDeleteted;
	std::string m_title;
	std::string m_savepath;
	static int m_iSelectStart;
	static int m_iPreSelect;

	// const
	HINSTANCE m_hInstance;

public:
	CLogWindow(const std::string* proto, const std::string* acc, CConComLog* pLines);
	~CLogWindow(void);
	void Create(HINSTANCE hInstance);
	void UpdateLog();
	void Activate();
	void UpdateFilter();
	void MarkDeleted();

public:
	bool IsShowing() const
	{
		return m_bDlgShowing;
	}
	void UnmarkDeleted()
	{
		m_bDeleteted = false;
	}
	void UpdateAcc()
	{
	}
	const CLogWindow& operator =(const CLogWindow&)
	{ _ASSERT(FALSE); }

private:
	// Window Message
	void OnSize(int nType, int width, int height);
	void OnMove(int x, int y);
	void OnContextMenu(HWND hWnd, int x, int y);
	void OnLogUpdated(bool bFull=false);
	void OnFilterUpdated();
	void OnClose();
	void OnDestroy();

	// Command ID
	void OnCopy();
	void OnSelectAll();
	void OnAllClear();
	void OnSave();
	void OnFilterSetting();

	void DeleteThisEntry();
	void ShowDlg();
	void CopyToClipboard(LPCTSTR str);
	void DestroyWindow();
	void UpdateListView(HWND hListWnd);
	void ResetListView(HWND hListWnd);
	void InsertLine(HWND hListWnd, int nPos, int no, const CLogLine &line);
	static void __cdecl FilterSettingDlgThread(void* hWnd);
	static LRESULT CALLBACK ListCtrlProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
	static int LogListHitTest(HWND hWnd, int x, int y);
	template <class Func>
		void ForEachListItem(UINT flags, Func &f);

private:
	void DeleteLine(HWND hListWnd, int nPos)
	{
		ListView_DeleteItem(hListWnd, nPos);
	}
	void UpdateWindowRect()
	{
		GetWindowRect(m_hDlgWnd, &m_rect);
	}
	void OnToggleAutoScroll()
	{
		COMLOG_SETTING.ToggleAutoScroll();
	}
	void OnToggleAutoStop()
	{
		COMLOG_SETTING.ToggleAutoStop();
	}

// I[o[Ch
public:
	BOOL DlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	BOOL InitDlgCtrl();
//	void GetDlgCtrlValue();
};
