#pragma once
#include <queue>

//***********************************************************

typedef std::queue<void*> AddrQueue;

//***********************************************************
class CThreadQueue
{
protected:
	CRITICAL_SECTION m_lock;
	HANDLE m_msgEvt;
	AddrQueue m_queue;

public:
	CThreadQueue(void)
	{
		m_msgEvt = CreateEvent(NULL, TRUE, FALSE, NULL);
		InitializeCriticalSection(&m_lock);
	}
	~CThreadQueue(void)
	{
		CloseHandle(m_msgEvt);
		DeleteCriticalSection(&m_lock);

		for( int i = m_queue.size(); i > 0; i-- ){
			delete m_queue.front();
			m_queue.pop();
		}
	}
	void PushAddr(void* addr)
	{
		Lock();
		m_queue.push(addr);
		Unlock();
		SetEvent(m_msgEvt);
	}
	void* PopAddr()
	{
		void* addr;

		WaitForSingleObject(m_msgEvt, INFINITE);
		Lock();
		_ASSERT( m_queue.size() != 0 );
		addr = m_queue.front();
		m_queue.pop();
		if( m_queue.size() == 0 ) ResetEvent(m_msgEvt);
		Unlock();

		return addr;
	}
	HANDLE GetWaitEventHandle()
	{
		return m_msgEvt;
	}

protected:
	void Lock()
	{
		EnterCriticalSection(&m_lock);
	}
	void Unlock()
	{
		LeaveCriticalSection(&m_lock);
	}
};
